#!/bin/sh
#
# Copyright (c) 2004 br1an <brian@ethernet.org>
#
#   Stephanie for OpenBSD 3.6
#   http://ethernet.org/~brian/Stephanie/
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

# Directory where Stephanie-3.6.tar.gz was extracted to.
STEPHANIE=~/Stephanie-3.6

#### ONLY CHANGE THESE IF YOU *REALLY* KNOW WHAT YOU'RE DOING! ####

# Directory where source tree is at. If your source tree is /usr/src,
# this needs to be '/usr'.
SRCPARENT=/usr

# Name of the source tree. If your source tree is /usr/src, this needs
# to be 'src'.
SRCTREENAME=src

######################################
### DO NOT CHANGE BELOW THIS LINE! ###
######################################

# Only root can use this.
if [ `whoami` != "root" ]; then
	echo 'Please run this script as root.'
	exit 1
fi

# Where the sources are.
SRCTREE=${SRCPARENT}/${SRCTREENAME}

/bin/cat << EOF

Stephanie for OpenBSD 3.6, br1an <brian@ethernet.org>
http://ethernet.org/~brian/Stephanie/

  Phase 1: Patching your kernel, synchronizing userland, compiling the new
           kernel, installing the new kernel and rebooting it.

  Phase 2: Recompiling libkvm and dependant programs, compiling and
           installing vexecctl, creating Vexec pseudo-device, installing
           userland privacy. Enter this phase only after you booted the
           new kernel.

EOF

# Verify.
echo -n "Parent directory for source tree: [${SRCPARENT}] "
read x
if [ $x ]; then
	SRCPARENT=$x
fi
echo -n "Source tree name: [${SRCTREENAME}] "
read x
if [ $x ]; then
	SRCTREENAME=$x
fi

# Where the sources are.
SRCTREE=${SRCPARENT}/${SRCTREENAME}

echo -n "Absolute path to source tree is \"${SRCTREE}\". Correct? [y/n] "
read x
if [ $x == 'n' ]; then
	echo "Aborting installation of Stephanie for OpenBSD 3.6."
	exit 1
fi

echo -n "Path to Stephanie for OpenBSD 3.6 files: [${STEPHANIE}] "
read x
if [ $x ]; then
	STEPHANIE=$x
fi

# Pick up where needed.
echo -n "Did you finish phase 1? [y/n]: "
read answer
if [ $answer == 'y' ]; then
	phase=2
else
	phase=1
fi

# Generate and install stephanie(7) and other documentation.
if [ $phase == 1 ]; then
	echo "[Phase 1] Installing documentation."
	cd ${STEPHANIE}/doc/ && /usr/bin/make clean && /usr/bin/make && /usr/bin/make install && /usr/bin/make clean
fi

# Patch source tree.
if [ $phase == 1 ]; then
echo "[Phase 1] Patching your source tree. (${SRCTREE})"
/usr/bin/patch -d ${SRCPARENT} -p0 < ${STEPHANIE}/Stephanie-3.6.patch
if [ $? != 0 ]; then
	/bin/cat << EOF

	An error occured during the process of patching your source tree
	with Stephanie for OpenBSD 3.6. There can be several causes to
	this:

	1) Your source tree is not the -release source tree. This might be
	   the case if you downloaded the -current or -stable branches
	   from the CVS server.

	2) Missing files. Make sure you have the complete source tree.

	3) Modified source tree. If you already applied a certain patch to
	   the tree, the diffs might not be able to fully apply due to
	   context changes. This can be if you stopped patching in the
	   middle aswell.

	If you think this is a bug with Stephanie for OpenBSD 3.6 and/or
	its installation script, please contact me and provide as many
	details as possible.

EOF

	exit 1
fi
fi

# Pseudo device in kernel config.
if [ $phase == 1 ]; then
	KERNELNAME=GENERIC
	echo -n "[Phase 1] Adding Vexec pseudo-device. MI Kernel name: [$KERNELNAME] "
	read x
	if [ $x ]; then
		KERNELNAME=$x
	fi
	echo "pseudo-device vexec	1	# Vexec pseudo-device" >> ${SRCTREE}/sys/conf/${KERNELNAME}
fi

# Copy Stephanie files to place.
if [ $phase == 1 ]; then
	echo "[Phase 1] Copying new files."
	/bin/cp -f ${STEPHANIE}/sys/{security.h,vexec.h} /usr/src/sys/sys/
	/bin/cp -f ${STEPHANIE}/kern/{kern_security.c,kern_vexec.c} /usr/src/sys/kern/
	/bin/cp -f ${STEPHANIE}/dev/vexec.c /usr/src/sys/dev/
fi

# Sync files.
if [ $phase == 1 ]; then
	echo "[Phase 1] Syncrhonizing userland with patched and new files."
	/bin/cp -f /usr/src/sys/sys/{conf.h,exec.h,proc.h,sysctl.h,vnode.h} /usr/include/sys/
	/bin/cp -f ${STEPHANIE}/sys/{security.h,vexec.h} /usr/include/sys/
fi

# Compile and install libkvm.
if [ $phase == 2 ]; then
	echo "[Phase 2] Recompiling and installing libkvm."
	cd /usr/src/lib/libkvm && /usr/bin/make clean && /usr/bin/make depend && /usr/bin/make && /usr/bin/make install && /usr/bin/make clean
fi

# Compile and install libkvm-dependant programs.
if [ $phase == 2 ]; then
	echo "[Phase 2] Recompiling and installing sysctl and libkvm-dependant programs."
	for prog in ${SRCTREE}/bin/ps/ ${SRCTREE}/sbin/sysctl/ ${SRCTREE}/usr.bin/systat/ ${SRCTREE}/usr.bin/vmstat/; do
		cd $prog && /usr/bin/make clean && /usr/bin/make && /usr/bin/make install && /usr/bin/make clean
	done
fi

# Regenerate sysctl(3) manpage.
if [ $phase == 2 ]; then
	/usr/bin/nroff -Tascii -mandoc /usr/src/lib/libc/gen/sysctl.3 > sysctl.cat3
	/usr/bin/install -c -o root -g bin -m 444 sysctl.cat3 /usr/share/man/cat3/sysctl.0
	/bin/rm -f sysctl.cat3
fi

# Compile and install vexecctl.
if [ $phase == 2 ]; then
	echo "[Phase 2] Compiling and installing vexecctl."
	cd ${STEPHANIE}/vexecctl && /usr/bin/make && /usr/bin/make install && /usr/bin/make clean
fi

# Create Vexec pseudo-device.
if [ $phase == 2 ]; then
	MACH=`machine`
	echo -n "[Phase 2] Creating Vexec pseudo-device. Machine: [$MACH] "
	read x
	if [ $x ]; then
		MACH=$x
	fi

	set +A ARCH	"alpha amd64 arm hp300 hppa i386 luna88k mac68k macppc mvme68k mvme88k mvmeppc sparc sparc64 vax"
	set +A DEVMAJOR	"58    83    99  53    40   84   53      53     79     53       53     56      52    53      75"
	index="0 1 2 3 4 5 6 7 8 9 10 11 12 13 14"
	found=0
	for i in $index; do
		if [ ${ARCH[i]} == $MACH ]; then
			found=1
			break
		fi
	done

	if [ $found == 1 ]; then
		/sbin/mknod -m 0600 /dev/vexec c ${DEVMAJOR[i]} 0
	else
		echo "[Phase 2] Unknown machine type. Skipping."
	fi
fi

# Userland privacy.
if [ $phase == 2 ]; then
	echo "[Phase 2] Setting up userland privacy."

	# Recompile programs.
	USERLAND_PRIVACY_PROGRAMS="finger last netstat w who"
	for i in $USERLAND_PRIVACY_PROGRAMS; do
		cd /usr/src/usr.bin/$i/
		/usr/bin/make clean && /usr/bin/make && /usr/bin/make install
	done

	# Permissions on existing logs.
	/bin/chmod 0660 /var/run/utmp
	/bin/chmod 0640 /var/log/wtmp
	/bin/chmod 0640 /var/log/lastlog

	# Patch newsyslog.conf, rc, mtree/special to maintain permissions.
	ERRMSG=""
	/usr/bin/patch -d /etc/ -p0 < ${STEPHANIE}/patches/newsyslog.conf.patch
	if [ $? != 0 ]; then
		ERRMSG="/etc/newsyslog.conf"
	fi
	/usr/bin/patch -d /etc/ -p0 < ${STEPHANIE}/patches/etc.rc.patch
	if [ $? != 0 ]; then
		if [ ERRMSG ]; then
			ERRMSG=$ERRMSG", /etc/rc"
		else
			ERRMSG="/etc/rc"
		fi
	fi
	/usr/bin/patch -d /etc/mtree/ -p0 < ${STEPHANIE}/patches/etc.mtree.special.patch
	if [ $? != 0 ]; then
		if [ ERRMSG ]; then
			ERRMSG=$ERRMSG", /etc/mtree/special"
		else
			ERRMSG="/etc/mtree/special"
		fi
	fi
	if [ $ERRMSG ]; then
	/bin/cat << EOF

	Stephanie for OpenBSD 3.6's installation program was unable to patch
	${ERRMSG}. This is not a problem, since there are very good the
	reason is changes you made yourself.

	To manually change it to comply with userland privacy, please
	apply the appropriate modifications to the files reported above.


	/etc/newsyslog.conf: Change (around line 14)

	  /var/log/wtmp                           644  7     *    168   ZB

	So it looks like:

	  /var/log/wtmp    root:utmp              640  7     *    168   ZB


	/etc/rc: Change (around line 200)

	  (cd /var/run && { rm -rf -- *; install -c -m 664 -g utmp /dev/null utmp; })

	To:

	  (cd /var/run && { rm -rf -- *; install -c -m 660 -g utmp /dev/null utmp; })


	/etc/mtree/special: Change (around line 166)

	  utmp		type=file mode=0664 uname=root gname=utmp

	To:

	  utmp		type=file mode=0660 uname=root gname=utmp


	This makes sure that logs we do filtering on maintain permissions after
	rotation/reboot.

EOF
fi

	# Change group.
	/bin/chgrp utmp /usr/bin/finger
	/bin/chgrp utmp /usr/bin/last
	/bin/chgrp utmp /usr/bin/w
	/bin/chgrp utmp /usr/bin/who

	# Set set-group-id bit.
	/bin/chmod 2555 /usr/bin/finger
	/bin/chmod 2555 /usr/bin/last
	/bin/chmod 2555 /usr/bin/w
	/bin/chmod 2555 /usr/bin/who
fi

# Config files changes.
if [ $phase == 2 ]; then
	VEXECPATH=/etc/vexec.conf

	/bin/cat << EOF
	The installation program can set a main Vexec config file to be
	loaded each time the system boots, before the securelevel is
	raised. If you would like that, enter the full path to the said
	file (you can create it in the future), or enter 'none' to skip.
EOF

	echo -n "Full path to vexec.conf file (or 'none'): [$VEXECPATH] "
	read x
	if [ $x ]; then
		# Check if 'none' or path.
		if [ $x != "none" ]; then
			VEXECPATH=$x
			echo "\n# Vexec fingerprints\nif [ -e ${VEXECPATH} ]; then\n\techo \"loading Vexec fingerprints from ${VEXECPATH}\"\n\t/usr/local/sbin/vexecctl ${VEXECPATH}\nfi" >> /etc/rc.securelevel
		fi
	else
		echo "\n# Vexec fingerprints\nif [ -e ${VEXECPATH} ]; then\n\techo \"loading Vexec fingerprints from ${VEXECPATH}\"\n\t/usr/local/sbin/vexecctl ${VEXECPATH}\nfi" >> /etc/rc.securelevel
	fi
fi

# End of phase notes
if [ $phase == 1 ]; then
	/bin/cat << EOF

	Phase 1 completed. Please recompile your kernel using the normal
	procedure and copy it to /bsd (or whatever) after backing up your old
	kernel.

	http://www.openbsd.org/faq/faq5.html#Building contains
	instructions for recompiling the modified kernel, in case you
	don't already know how to do so.

	Once done, reboot your system and execute this installation
	script again - this time, choosing to enter phase 2.

EOF
else
	/bin/cat << EOF

	Phase 2 completed. Your system now has Stephanie for OpenBSD 3.6
	installed. Please read the accompanied manuals: stephanie(7),
	vexecctl(8), vexec.conf(5).

	You may want to run the supplied 'fpgen.sh' script to create
	a /etc/vexec.conf. Remember - you can load it only when the
	system is single-user.

	Also, consider using the Stephanie settings of your choice by
	placing them in /etc/sysctl.conf.

	Thanks for installing Stephanie for OpenBSD 3.6! If you encounter
	any problems, please mail me to <brian@ethernet.org>. Stephanie's
	official website is http://ethernet.org/~brian/Stephanie/ - there
	you can always find the latest news and releases of Stephanie.

EOF
fi
