/*
 * Copyright (c) 2004 br1an <brian@ethernet.org>
 *
 *   Stephanie for OpenBSD 3.6
 *   http://ethernet.org/~brian/Stephanie/
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef _VEXECCTL_H_
#define _VEXECCTL_H_

/*
 * Little code to manage the loading of device information to the kernel.
 * This will hold the gathered information about referenced devices in the
 * config file and how many fingerprinted files there are on each device.
 * We're pulling vexec_hash_param because that's the struct we'll be
 * copying to the kernel and we don't want to waste the bytes for the
 * pointer.
 */
CIRCLEQ_HEAD(vexec_ups, vexec_up) params_list;
struct vexec_up {
        struct vexec_hash_param vu_param;
        CIRCLEQ_ENTRY(vexec_up) vu_list;
};

/*
 * Used to track filename/line number per included file, as (f)lex don't
 * do it already. Pointers to those who maintain it:
 *  - Make the yylineno bigger than an integer.
 *  - Add it as a per-state (or -buffer or -context) variable and not
 *    one global.
 *  - An option like "yyfilename" would be nice too. :)
 *
 * These can make (f)lex slower (in numbers, of course), but could be
 * very useful to have. I think...
 */
struct my_stack {
	char *name;
	u_int line;
	struct my_stack *next;
	struct my_stack *prev;
};

/* Ghetto macro to do some recursive freeing of our file stack. */
#define filestack_free(s)					\
		while (s) {					\
			if (s->prev) {				\
				s = s->prev;			\
				if (s->next->name) {		\
					free(s->next->name);	\
					s->next->name = NULL;	\
				}				\
				free(s->next);			\
				s->next = NULL;			\
			} else {				\
				if (s->name) {			\
					free(s->name);		\
					s->next->name = NULL;	\
				}				\
				free(s);			\
				s = NULL;			\
			}					\
		}						\

extern struct my_stack *filestack;
extern int fd, no_mem, phase, verbose;
extern FILE *yyin;

int yywrap(void);
int yylex(void);
int yyparse(void);
void yyerror(const char *);
FILE *openlock(const char *);
struct vexec_up *dev_lookup(dev_t);
struct vexec_up *dev_add(dev_t);
int phase1_preload(void);
void phase2_load(void);
int convert(char *, u_int, u_char *);

#endif /* _VEXECCTL_H_ */
